#!/usr/bin/env python

# Convert an iCal file (such as that fetched from a server)
# into a local users crontab.

import ConfigParser
import urllib2
import datetime
import sys
import os
import re

import iCal
import pycrontab

def get_calendar(calname, calurl):
	# Fetch a calendar from the given url... or
	# read it from a file.
	cal = None
	if calurl.startswith('/'):
		try:
			cal = ICalReader([calurl])
		except IOError, e:
			print "Unable to open file %s: %s" % (calurl, e)
	else:
		try:
			f = urllib2.urlopen(calurl)
			cal = iCal.ICalReader(inputlines = f.readlines())
		except urllib2.HTTPError, e:
			print "Unable to download %s: %s" % (calurl, e)

	return cal

def get_calendar_commands(calname, config):
	# Fetch commands from the configuration file.
	commands = {}
	
	try:
		# First, the space seperated list of commands.
		cmdlist = config.get("commands", calname)	
		cmdlist = cmdlist.split()

		# Each command has a section in the config,
		# "cmd-foo". Inside that must be match and oncommand,
		# and optionally offcommand.
		for cmd in cmdlist:
			sectionName = "cmd-" + cmd
			newcmd = {}
			match = None

			# Attempt to get the required options.
			try:
				match = re.compile(config.get(sectionName, "match"), re.IGNORECASE)
				newcmd['oncmd'] = config.get(sectionName, "oncommand")
			except ConfigParser.NoOptionError:
				# Oops! This command will be skipped.
				match = None
				print "Missing either match or oncommand for command '%s'" % cmd

			# And this one is optional.
			try:
				newcmd['offcmd'] = config.get(sectionName, "offcommand")
			except ConfigParser.NoOptionError:
				# Ignore; this parameter is optional.
				pass

			# The hash key is the re object.
			if match != None:
				commands[match] = newcmd

	except ConfigParser.NoOptionError:
		print "No commands for this calendar!"
	
	return commands

def add_to_crontab(crontab, matches, command, event, date, calendar):
	# Do we need the off command?
	# Set a flag to be used later.
	needoff = command.has_key("offcmd")

	# Determine the time and date required.
	# For the on command, this is simple...
	ontime = date.replace(hour = event.startDate.hour, minute = event.startDate.minute,
				second = 0)
	
	# Hack to fix google calendar suddenly not supplying a duration...
	if not event.duration:
		event.duration = (event.endDate - event.startDate).seconds

	if needoff:
		# For the off command, add duration to the
		# start time. If this rolls over a day,
		# it will be handled.
		offtime = ontime
		offtime += datetime.timedelta(0, event.duration)
	
	# Escape all the parameters ready for the shell.
	params = matches.groupdict()
	for key in params:
		params[key] = re.escape(params[key])

	# Add a few special groups to the substitution groups.
	params["calendar"] = calendar
	params["date"] = ontime.strftime("%Y-%m-%d")
	params["time"] = ontime.strftime("%H:%M:%S")
	params["duration"] = "%d" % event.duration
	if needoff:
		params["offdate"] = offtime.strftime("%Y-%m-%d")
		params["offtime"] = offtime.strftime("%H:%M:%S")

	# Generate the commands.
	oncmd = command["oncmd"] % params
	if needoff:
		offcmd = command["offcmd"] % params

	crontab.addentry(ontime, oncmd)
	if needoff:
		crontab.addentry(offtime, offcmd)

### Actually start doing things...

# Open the configuration
config = ConfigParser.RawConfigParser()
config.read("icaltasks.ini");

# Read the list of calendars
try:
	calendars = config.items("calendars")
except ConfigParser.NoSectionError:
	print "Configuration missing [calendars] section.\n"
	sys.exit(1)

# Grab other settings from config.
def setting_or_default(config, section, name, default):
	try:
		return config.read(section, name)
	except:
		return default

DAYSAHEAD = setting_or_default(config, "global", "daysahead", 7)

# Read current crontab.
crontab = pycrontab.PyCrontab()
crontab.readtab(stripbegin = "# ICALTOCRON-START", stripend = "# ICALTOCRON-END\n")
	
crontab.addrawline("")
crontab.addrawline("# ICALTOCRON-START")
crontab.addrawline("# Automatically generated - if you edit, any changes will be removed")
crontab.addrawline("# next time icaltocron is run.")

# Now iterate over the calendars.
for calendar in calendars:
	# Go and fetch the ics file.
	calname = calendar[0]
	calurl = calendar[1]

	cal = get_calendar(calname, calurl)

	# Grab a list of tasks for this calendar.
	commands = get_calendar_commands(calname, config)

	# Look for events occuring over the next N days.
	fordate = datetime.datetime.today()
	nextday = datetime.timedelta(1)
	enddate = datetime.datetime.today() + datetime.timedelta(DAYSAHEAD)

	# Loop until we have searched for N days...
	while fordate < enddate:
		# Find each event occuring on that day.
		for event in cal.eventsFor(fordate):
			# For each event, see if we have commands that match that event.
			for cmd, params in commands.iteritems():
				# Match it against a RE
				result = cmd.match(event.summary.strip())
				# If matching...
				if result != None:
					# Ok, found event. Write to crontab.
					add_to_crontab(crontab, result, params,	
							event, fordate, calname)

		# Look at the next day.
		fordate += nextday

# Now we have a list of tasks from all the calendars. Write it!
crontab.addrawline("# ICALTOCRON-END\n")
crontab.writetab()
